<?php

declare(strict_types=1);

namespace Drupal\Tests\smart_migrate_cli\Traits;

use Drupal\Core\Database\Database;
use Drupal\Tests\migmag\Traits\MigMagMigrationTestDatabaseTrait;

/**
 * Advanced version of the MigMag trait resolving issue with DB driver value.
 */
trait AdvancedMigMagMigrationTestDatabaseTrait {

  use MigMagMigrationTestDatabaseTrait;

  /**
   * {@inheritdoc}
   */
  protected function installParameters(): array {
    $installParams = parent::installParameters();
    unset($installParams['forms']['install_settings_form']['driver']);
    return $installParams;
  }

  /**
   * {@inheritdoc}
   */
  protected function createSourceMigrationConnection(string $source_db_key = 'migrate') {
    // If the backup already exists, something went terribly wrong.
    // This case is possible, because database connection info is a static
    // global state construct on the Database class, which at least persists
    // for all test methods executed in one PHP process.
    if (Database::getConnectionInfo("simpletest_original_$source_db_key")) {
      throw new \RuntimeException("Bad Database connection state: 'simpletest_original_$source_db_key' connection key already exists. Broken test?");
    }

    // Clone the current connection and replace the current prefix.
    $connection_info = Database::getConnectionInfo($source_db_key);
    if ($connection_info) {
      Database::renameConnection($source_db_key, "simpletest_original_$source_db_key");
    }
    $connection_info = Database::getConnectionInfo('default');
    // Find out the right version here.
    $driverList = is_callable([Database::class, 'getDriverList'])
      ? Database::getDriverList()
      : NULL;
    foreach ($connection_info as $target => $value) {
      $prefix = is_array($value['prefix']) ? $value['prefix']['default'] : $value['prefix'];
      if ($driverList) {
        $currentDriver = $driverList->get($value['namespace']);
        $connection_info[$target]['driver'] = $currentDriver->getDriverName();
      }
      // Simpletest uses 7 character prefixes at most so this can't cause
      // collisions.
      if (is_string($connection_info[$target]['prefix'])) {
        $connection_info[$target]['prefix'] = $prefix . '0';
        continue;
      }

      $connection_info[$target]['prefix']['default'] = $prefix . '0';
      // Add the original simpletest prefix so SQLite can attach its database.
      // @see \Drupal\Core\Database\Driver\sqlite\Connection::init()
      $connection_info[$target]['prefix'][$value['prefix']['default']] = $prefix;
      break;
    }
    Database::addConnectionInfo($source_db_key, 'default', $connection_info['default']);

    // If we are in a functional test, we might use Drush for testing things.
    // So we write the connection info into settings.php.
    if (is_callable([$this, 'writeSettings'])) {
      $this->writeSettings([
        'databases' => [
          "{$source_db_key}']['default" => (object) [
            'value' => $connection_info['default'],
            'required' => TRUE,
          ],
        ],
      ]);
    }

    if ($source_db_key !== 'migrate') {
      \Drupal::state()->set('migmag.test_db', [
        'key' => $source_db_key,
        'target' => 'default',
      ]);
      \Drupal::state()->set('migrate.fallback_state_key', 'migmag.test_db');
    }
  }

}
